import sys
import typing
import bpy.types


def abspath(path,
            start: typing.Union[bytes, str] = None,
            library: 'bpy.types.Library' = None):
    ''' Returns the absolute path relative to the current blend file using the "//" prefix.

    :param start: Relative to this path, when not set the current filename is used.
    :type start: typing.Union[bytes, str]
    :param library: The library this path is from. This is only included for convenience, when the library is not None its path replaces *start*.
    :type library: 'bpy.types.Library'
    '''

    pass


def basename(path):
    ''' Equivalent to os.path.basename, but skips a "//" prefix. Use for Windows compatibility.

    '''

    pass


def clean_name(name, replace='_'):
    ''' Returns a name with characters replaced that may cause problems under various circumstances, such as writing to a file. All characters besides A-Z/a-z, 0-9 are replaced with "_" or the *replace* argument if defined.

    '''

    pass


def display_name(name, *, has_ext=True):
    ''' Creates a display string from name to be used menus and the user interface. Capitalize the first letter in all lowercase names, mixed case names are kept as is. Intended for use with filenames and module names.

    '''

    pass


def display_name_from_filepath(name):
    ''' Returns the path stripped of directory and extension, ensured to be utf8 compatible.

    '''

    pass


def display_name_to_filepath(name):
    ''' Performs the reverse of display_name using literal versions of characters which aren't supported in a filepath.

    '''

    pass


def ensure_ext(filepath, ext: str, case_sensitive: bool = False):
    ''' Return the path with the extension added if it is not already set.

    :param ext: The extension to check for, can be a compound extension. Should start with a dot, such as '.blend' or '.tar.gz'.
    :type ext: str
    :param case_sensitive: Check for matching case when comparing extensions.
    :type case_sensitive: bool
    '''

    pass


def is_subdir(path: typing.Union[bytes, str], directory):
    ''' Returns true if *path* in a subdirectory of *directory*. Both paths must be absolute.

    :param path: An absolute path.
    :type path: typing.Union[bytes, str]
    '''

    pass


def module_names(path: str, recursive: bool = False) -> list:
    ''' Return a list of modules which can be imported from *path*.

    :param path: a directory to scan.
    :type path: str
    :param recursive: Also return submodule names for packages.
    :type recursive: bool
    :return: a list of string pairs (module_name, module_file).
    '''

    pass


def native_pathsep(path):
    ''' Replace the path separator with the systems native os.sep .

    '''

    pass


def reduce_dirs(dirs: list) -> list:
    ''' Given a sequence of directories, remove duplicates and any directories nested in one of the other paths. (Useful for recursive path searching).

    :param dirs: Sequence of directory paths.
    :type dirs: list
    :return: A unique list of paths.
    '''

    pass


def relpath(path: typing.Union[bytes, str],
            start: typing.Union[bytes, str] = None):
    ''' Returns the path relative to the current blend file using the "//" prefix.

    :param path: An absolute path.
    :type path: typing.Union[bytes, str]
    :param start: Relative to this path, when not set the current filename is used.
    :type start: typing.Union[bytes, str]
    '''

    pass


def resolve_ncase(path):
    ''' Resolve a case insensitive path on a case sensitive system, returning a string with the path if found else return the original path.

    '''

    pass



import sys
import typing


def add_target():
    ''' Add a target to the constraint

    '''

    pass


def childof_clear_inverse(constraint: str = "",
                          owner: typing.Union[str, int] = 'OBJECT'):
    ''' Clear inverse correction for ChildOf constraint

    :param constraint: Constraint, Name of the constraint to edit
    :type constraint: str
    :param owner: Owner, The owner of this constraint * OBJECT Object, Edit a constraint on the active object. * BONE Bone, Edit a constraint on the active bone.
    :type owner: typing.Union[str, int]
    '''

    pass


def childof_set_inverse(constraint: str = "",
                        owner: typing.Union[str, int] = 'OBJECT'):
    ''' Set inverse correction for ChildOf constraint

    :param constraint: Constraint, Name of the constraint to edit
    :type constraint: str
    :param owner: Owner, The owner of this constraint * OBJECT Object, Edit a constraint on the active object. * BONE Bone, Edit a constraint on the active bone.
    :type owner: typing.Union[str, int]
    '''

    pass


def delete():
    ''' Remove constraint from constraint stack

    '''

    pass


def disable_keep_transform():
    ''' Set the influence of this constraint to zero while trying to maintain the object's transformation. Other active constraints can still influence the final transformation

    '''

    pass


def followpath_path_animate(constraint: str = "",
                            owner: typing.Union[str, int] = 'OBJECT',
                            frame_start: int = 1,
                            length: int = 100):
    ''' Add default animation for path used by constraint if it isn't animated already

    :param constraint: Constraint, Name of the constraint to edit
    :type constraint: str
    :param owner: Owner, The owner of this constraint * OBJECT Object, Edit a constraint on the active object. * BONE Bone, Edit a constraint on the active bone.
    :type owner: typing.Union[str, int]
    :param frame_start: Start Frame, First frame of path animation
    :type frame_start: int
    :param length: Length, Number of frames that path animation should take
    :type length: int
    '''

    pass


def limitdistance_reset(constraint: str = "",
                        owner: typing.Union[str, int] = 'OBJECT'):
    ''' Reset limiting distance for Limit Distance Constraint

    :param constraint: Constraint, Name of the constraint to edit
    :type constraint: str
    :param owner: Owner, The owner of this constraint * OBJECT Object, Edit a constraint on the active object. * BONE Bone, Edit a constraint on the active bone.
    :type owner: typing.Union[str, int]
    '''

    pass


def move_down(constraint: str = "", owner: typing.Union[str, int] = 'OBJECT'):
    ''' Move constraint down in constraint stack

    :param constraint: Constraint, Name of the constraint to edit
    :type constraint: str
    :param owner: Owner, The owner of this constraint * OBJECT Object, Edit a constraint on the active object. * BONE Bone, Edit a constraint on the active bone.
    :type owner: typing.Union[str, int]
    '''

    pass


def move_up(constraint: str = "", owner: typing.Union[str, int] = 'OBJECT'):
    ''' Move constraint up in constraint stack

    :param constraint: Constraint, Name of the constraint to edit
    :type constraint: str
    :param owner: Owner, The owner of this constraint * OBJECT Object, Edit a constraint on the active object. * BONE Bone, Edit a constraint on the active bone.
    :type owner: typing.Union[str, int]
    '''

    pass


def normalize_target_weights():
    ''' Normalize weights of all target bones

    '''

    pass


def objectsolver_clear_inverse(constraint: str = "",
                               owner: typing.Union[str, int] = 'OBJECT'):
    ''' Clear inverse correction for ObjectSolver constraint

    :param constraint: Constraint, Name of the constraint to edit
    :type constraint: str
    :param owner: Owner, The owner of this constraint * OBJECT Object, Edit a constraint on the active object. * BONE Bone, Edit a constraint on the active bone.
    :type owner: typing.Union[str, int]
    '''

    pass


def objectsolver_set_inverse(constraint: str = "",
                             owner: typing.Union[str, int] = 'OBJECT'):
    ''' Set inverse correction for ObjectSolver constraint

    :param constraint: Constraint, Name of the constraint to edit
    :type constraint: str
    :param owner: Owner, The owner of this constraint * OBJECT Object, Edit a constraint on the active object. * BONE Bone, Edit a constraint on the active bone.
    :type owner: typing.Union[str, int]
    '''

    pass


def remove_target(index: int = 0):
    ''' Remove the target from the constraint

    :param index: index
    :type index: int
    '''

    pass


def stretchto_reset(constraint: str = "",
                    owner: typing.Union[str, int] = 'OBJECT'):
    ''' Reset original length of bone for Stretch To Constraint

    :param constraint: Constraint, Name of the constraint to edit
    :type constraint: str
    :param owner: Owner, The owner of this constraint * OBJECT Object, Edit a constraint on the active object. * BONE Bone, Edit a constraint on the active bone.
    :type owner: typing.Union[str, int]
    '''

    pass



import sys
import typing


def autopack_toggle():
    ''' Automatically pack all external files into the .blend file

    '''

    pass


def bookmark_add():
    ''' Add a bookmark for the selected/active directory

    '''

    pass


def bookmark_cleanup():
    ''' Delete all invalid bookmarks

    '''

    pass


def bookmark_delete(index: int = -1):
    ''' Delete selected bookmark

    :param index: Index
    :type index: int
    '''

    pass


def bookmark_move(direction: typing.Union[str, int] = 'TOP'):
    ''' Move the active bookmark up/down in the list

    :param direction: Direction, Direction to move the active bookmark towards * TOP Top, Top of the list. * UP Up. * DOWN Down. * BOTTOM Bottom, Bottom of the list.
    :type direction: typing.Union[str, int]
    '''

    pass


def bookmark_toggle():
    ''' Toggle bookmarks display

    '''

    pass


def cancel():
    ''' Cancel loading of selected file

    '''

    pass


def delete():
    ''' Delete selected files

    '''

    pass


def directory_new(directory: str = "", open: bool = False):
    ''' Create a new directory

    :param directory: Directory, Name of new directory
    :type directory: str
    :param open: Open, Open new directory
    :type open: bool
    '''

    pass


def execute(need_active: bool = False):
    ''' Execute selected file

    :param need_active: Need Active, Only execute if there's an active selected file in the file list
    :type need_active: bool
    '''

    pass


def filenum(increment: int = 1):
    ''' Increment number in filename

    :param increment: Increment
    :type increment: int
    '''

    pass


def filepath_drop(filepath: str = "Path"):
    ''' Undocumented contribute <https://developer.blender.org/T51061>

    :type filepath: str
    '''

    pass


def find_missing_files(
        find_all: bool = False,
        directory: str = "",
        filter_blender: bool = False,
        filter_backup: bool = False,
        filter_image: bool = False,
        filter_movie: bool = False,
        filter_python: bool = False,
        filter_font: bool = False,
        filter_sound: bool = False,
        filter_text: bool = False,
        filter_btx: bool = False,
        filter_collada: bool = False,
        filter_alembic: bool = False,
        filter_folder: bool = False,
        filter_blenlib: bool = False,
        filemode: int = 9,
        display_type: typing.Union[str, int] = 'DEFAULT',
        sort_method: typing.Union[str, int] = 'FILE_SORT_ALPHA'):
    ''' Try to find missing external files

    :param find_all: Find All, Find all files in the search path (not just missing)
    :type find_all: bool
    :param directory: Directory, Directory of the file
    :type directory: str
    :param filter_blender: Filter .blend files
    :type filter_blender: bool
    :param filter_backup: Filter .blend files
    :type filter_backup: bool
    :param filter_image: Filter image files
    :type filter_image: bool
    :param filter_movie: Filter movie files
    :type filter_movie: bool
    :param filter_python: Filter python files
    :type filter_python: bool
    :param filter_font: Filter font files
    :type filter_font: bool
    :param filter_sound: Filter sound files
    :type filter_sound: bool
    :param filter_text: Filter text files
    :type filter_text: bool
    :param filter_btx: Filter btx files
    :type filter_btx: bool
    :param filter_collada: Filter COLLADA files
    :type filter_collada: bool
    :param filter_alembic: Filter Alembic files
    :type filter_alembic: bool
    :param filter_folder: Filter folders
    :type filter_folder: bool
    :param filter_blenlib: Filter Blender IDs
    :type filter_blenlib: bool
    :param filemode: File Browser Mode, The setting for the file browser mode to load a .blend file, a library or a special file
    :type filemode: int
    :param display_type: Display Type * DEFAULT Default, Automatically determine display type for files. * LIST_SHORT Short List, Display files as short list. * LIST_LONG Long List, Display files as a detailed list. * THUMBNAIL Thumbnails, Display files as thumbnails.
    :type display_type: typing.Union[str, int]
    :param sort_method: File sorting mode * FILE_SORT_ALPHA Sort alphabetically, Sort the file list alphabetically. * FILE_SORT_EXTENSION Sort by extension, Sort the file list by extension/type. * FILE_SORT_TIME Sort by time, Sort files by modification time. * FILE_SORT_SIZE Sort by size, Sort files by size.
    :type sort_method: typing.Union[str, int]
    '''

    pass


def hidedot():
    ''' Toggle hide hidden dot files

    '''

    pass


def highlight():
    ''' Highlight selected file(s)

    '''

    pass


def make_paths_absolute():
    ''' Make all paths to external files absolute

    '''

    pass


def make_paths_relative():
    ''' Make all paths to external files relative to current .blend

    '''

    pass


def next():
    ''' Move to next folder

    '''

    pass


def pack_all():
    ''' Pack all used external files into the .blend

    '''

    pass


def pack_libraries():
    ''' Pack all used Blender library files into the current .blend

    '''

    pass


def parent():
    ''' Move to parent directory

    '''

    pass


def previous():
    ''' Move to previous folder

    '''

    pass


def refresh():
    ''' Refresh the file list

    '''

    pass


def rename():
    ''' Rename file or file directory

    '''

    pass


def report_missing_files():
    ''' Report all missing external files

    '''

    pass


def reset_recent():
    ''' Reset Recent files

    '''

    pass


def select(extend: bool = False, fill: bool = False, open: bool = True):
    ''' Activate/select file

    :param extend: Extend, Extend selection instead of deselecting everything first
    :type extend: bool
    :param fill: Fill, Select everything beginning with the last selection
    :type fill: bool
    :param open: Open, Open a directory when selecting it
    :type open: bool
    '''

    pass


def select_all(action: typing.Union[str, int] = 'TOGGLE'):
    ''' Select or deselect all files

    :param action: Action, Selection action to execute * TOGGLE Toggle, Toggle selection for all elements. * SELECT Select, Select all elements. * DESELECT Deselect, Deselect all elements. * INVERT Invert, Invert selection of all elements.
    :type action: typing.Union[str, int]
    '''

    pass


def select_bookmark(dir: str = ""):
    ''' Select a bookmarked directory

    :param dir: Dir
    :type dir: str
    '''

    pass


def select_box(xmin: int = 0,
               xmax: int = 0,
               ymin: int = 0,
               ymax: int = 0,
               wait_for_input: bool = True,
               mode: typing.Union[str, int] = 'SET'):
    ''' Activate/select the file(s) contained in the border

    :param xmin: X Min
    :type xmin: int
    :param xmax: X Max
    :type xmax: int
    :param ymin: Y Min
    :type ymin: int
    :param ymax: Y Max
    :type ymax: int
    :param wait_for_input: Wait for Input
    :type wait_for_input: bool
    :param mode: Mode * SET Set, Set a new selection. * ADD Extend, Extend existing selection. * SUB Subtract, Subtract existing selection.
    :type mode: typing.Union[str, int]
    '''

    pass


def select_walk(direction: typing.Union[str, int] = 'UP',
                extend: bool = False,
                fill: bool = False):
    ''' Select/Deselect files by walking through them

    :param direction: Walk Direction, Select/Deselect file in this direction
    :type direction: typing.Union[str, int]
    :param extend: Extend, Extend selection instead of deselecting everything first
    :type extend: bool
    :param fill: Fill, Select everything beginning with the last selection
    :type fill: bool
    '''

    pass


def smoothscroll():
    ''' Smooth scroll to make editable file visible

    '''

    pass


def unpack_all(method: typing.Union[str, int] = 'USE_LOCAL'):
    ''' Unpack all files packed into this .blend to external ones

    :param method: Method, How to unpack
    :type method: typing.Union[str, int]
    '''

    pass


def unpack_item(method: typing.Union[str, int] = 'USE_LOCAL',
                id_name: str = "",
                id_type: int = 19785):
    ''' Unpack this file to an external file

    :param method: Method, How to unpack
    :type method: typing.Union[str, int]
    :param id_name: ID name, Name of ID block to unpack
    :type id_name: str
    :param id_type: ID Type, Identifier type of ID block
    :type id_type: int
    '''

    pass


def unpack_libraries():
    ''' Unpack all used Blender library files from this .blend file

    '''

    pass



import sys
import typing
import bpy_types


class UnifiedPaintPanel:
    def paint_settings(self, context):
        ''' 

        '''
        pass

    def prop_unified_color(self, parent, context, brush, prop_name, text):
        ''' 

        '''
        pass

    def prop_unified_color_picker(self, parent, context, brush, prop_name,
                                  value_slider):
        ''' 

        '''
        pass

    def prop_unified_size(self, parent, context, brush, prop_name, icon, text,
                          slider):
        ''' 

        '''
        pass

    def prop_unified_strength(self, parent, context, brush, prop_name, icon,
                              text, slider):
        ''' 

        '''
        pass

    def prop_unified_weight(self, parent, context, brush, prop_name, icon,
                            text, slider):
        ''' 

        '''
        pass

    def unified_paint_settings(self, parent, context):
        ''' 

        '''
        pass


class VIEW3D_MT_tools_projectpaint_clone(bpy_types.Menu, bpy_types._GenericUI):
    bl_label = None
    ''' '''

    bl_rna = None
    ''' '''

    id_data = None
    ''' '''

    def append(self, draw_func):
        ''' 

        '''
        pass

    def as_pointer(self):
        ''' 

        '''
        pass

    def bl_rna_get_subclass(self):
        ''' 

        '''
        pass

    def bl_rna_get_subclass_py(self):
        ''' 

        '''
        pass

    def draw(self, context):
        ''' 

        '''
        pass

    def draw_collapsible(self, context, layout):
        ''' 

        '''
        pass

    def draw_preset(self, _context):
        ''' 

        '''
        pass

    def driver_add(self):
        ''' 

        '''
        pass

    def driver_remove(self):
        ''' 

        '''
        pass

    def get(self):
        ''' 

        '''
        pass

    def is_extended(self):
        ''' 

        '''
        pass

    def is_property_hidden(self):
        ''' 

        '''
        pass

    def is_property_overridable_library(self):
        ''' 

        '''
        pass

    def is_property_readonly(self):
        ''' 

        '''
        pass

    def is_property_set(self):
        ''' 

        '''
        pass

    def items(self):
        ''' 

        '''
        pass

    def keyframe_delete(self):
        ''' 

        '''
        pass

    def keyframe_insert(self):
        ''' 

        '''
        pass

    def keys(self):
        ''' 

        '''
        pass

    def path_from_id(self):
        ''' 

        '''
        pass

    def path_menu(self, searchpaths, operator, props_default, prop_filepath,
                  filter_ext, filter_path, display_name, add_operator):
        ''' 

        '''
        pass

    def path_resolve(self):
        ''' 

        '''
        pass

    def pop(self):
        ''' 

        '''
        pass

    def prepend(self, draw_func):
        ''' 

        '''
        pass

    def property_overridable_library_set(self):
        ''' 

        '''
        pass

    def property_unset(self):
        ''' 

        '''
        pass

    def remove(self, draw_func):
        ''' 

        '''
        pass

    def type_recast(self):
        ''' 

        '''
        pass

    def values(self):
        ''' 

        '''
        pass


def brush_basic_gpencil_paint_settings(layout, _context, brush, compact):
    ''' 

    '''

    pass


def brush_basic_gpencil_sculpt_settings(layout, context, brush, compact):
    ''' 

    '''

    pass


def brush_basic_gpencil_weight_settings(layout, _context, brush, compact):
    ''' 

    '''

    pass


def brush_basic_sculpt_settings(layout, context, brush, compact):
    ''' 

    '''

    pass


def brush_basic_texpaint_settings(layout, context, brush, compact):
    ''' 

    '''

    pass


def brush_basic_vpaint_settings(layout, context, brush, compact):
    ''' 

    '''

    pass


def brush_basic_wpaint_settings(layout, context, brush, compact):
    ''' 

    '''

    pass


def brush_mask_texture_settings(layout, brush):
    ''' 

    '''

    pass


def brush_texpaint_common(panel, context, layout, brush, _settings, projpaint):
    ''' 

    '''

    pass


def brush_texpaint_common_clone(_panel, context, layout, _brush, settings,
                                projpaint):
    ''' 

    '''

    pass


def brush_texpaint_common_color(_panel, context, layout, brush, _settings,
                                projpaint):
    ''' 

    '''

    pass


def brush_texpaint_common_gradient(_panel, context, layout, brush, _settings,
                                   projpaint):
    ''' 

    '''

    pass


def brush_texpaint_common_options(_panel, _context, layout, brush, _settings,
                                  projpaint):
    ''' 

    '''

    pass


def brush_texture_settings(layout, brush, sculpt):
    ''' 

    '''

    pass



import sys
import typing


def color_add():
    ''' Add new color to active palette

    '''

    pass


def color_delete():
    ''' Remove active color from palette

    '''

    pass


def new():
    ''' Add new palette

    '''

    pass
